/**<pre>
 * Title: iSIM
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux
 * @version 1.0
 * </pre>
 */

package agent.autonomous;

import agent.*;
import controller.*;
import gui.*;
import map.*;
import transport.*;
import transport.protocol.*;

import java.awt.*;
import java.awt.event.*;
import java.util.Date;
import java.io.*;
import java.util.*;


/* ====================================================================== */
/** This class represents a fire.  It hurts agents near it, and it can be
 *  hurt/extinguished with water.  Agents and fire have health.
 *
 *
 */
public class Fire extends Autonomous implements SocketReceiverListener{

	/** The simulation controller, only used to determine our simulation
	    environment boundaries. */
	private Controller controller;

	/** A thread used for this robot to send wireless communication message. */
	private Calling calling;

	private boolean kill = false;

	/* ====================================================================== */
	/** This constructor sets up the information needed for the robot.
	 *  Naturally the robot must know its own abilities.  The simulation
	 *  controller is needed so that it can be given to the mediator class,
	 *  Autonomous.java, which serves as an inbetween for a robot and the
	 *  simulator.
	 *
	 *  @param  c   The iSIM controller.
	 *  @param  loc  The location of the agent.
	 *  @param  port  The communication port need to speak with iSIM.
	 *  @param  range   The wireless max range.
	 *
	 */
	public Fire(Controller c, Point loc, int port, int range){

		// Super our agent/user information.
		super(c, loc, port, range, "fire");

		// Simulation controller
		controller = c;
		setHealth(100);

		calling = new Calling();
		calling.start();


	} //Constructor


	public void killFire(){
		kill = true;
	}


	// Use this method to receive new location data.
	public void senseLocation(Point location, double heading){}

	public void senseMessage(HelpMessage msg){}



	public void senseMessage(Message msg){

		//check for water message..
		//and hurt ourselves..
		if( msg instanceof WaterMessage ){

			double s = ((WaterMessage)msg).getStrength();

			//Then hurt fire some degree
			int health = getHealth();
			health -= .10*s;
			setHealth(health);
			controller.fireControllerChange();

			//Delete if health is zero
			if( health < 0 ){
				controller.removeUser(this);
				controller.fireControllerChange();
			}

		}

	}



	// Returns the current location of this autonomous robot.
	public Point getPoint(){

		return this.getLocation();
	}


	public void spread(Vector rooms ){

        System.out.println( "Fire trying to spread..." );
        Point point1 = new Point( getPoint().x + 50, getPoint().y );
        Point point2 = new Point( getPoint().x - 50, getPoint().y );
        Point point3 = new Point( getPoint().x, getPoint().y + 50 );
        Point point4 = new Point( getPoint().x, getPoint().y - 50 );

        for( int i=0;i < 10; i++ ){

            Room room1 = controller.getChartComp().isRoom( point1 );

            if( room1 != null && !rooms.contains( room1 ) )
            {
                if( controller.getChartComp().isRoom( new Point( point1.x + 75, point1.y ) ) == room1 )
                    controller.addFire( getPort() + (i + 10), getRange(), new Point( point1.x + 75, point1.y ) );
                else
                    controller.addFire( getPort() + (i + 10), getRange(), point1 );
                return;
            }

            room1 = controller.getChartComp().isRoom( point2 );
            if( room1 != null && !rooms.contains( room1 ) )
            {
				if( controller.getChartComp().isRoom( new Point( point2.x - 75, point2.y ) ) == room1 )
					controller.addFire( getPort() + (i + 10), getRange(), new Point( point2.x - 75, point2.y ) );
				else
					controller.addFire( getPort() + (i + 10), getRange(), point2 );
				return;
			}

			room1 = controller.getChartComp().isRoom( point3 );
			if( room1 != null && !rooms.contains( room1 ) )
			{
				if( controller.getChartComp().isRoom( new Point( point3.x, point3.y + 75 ) ) == room1 )
					controller.addFire( getPort() + (i + 10), getRange(), new Point( point3.x, point3.y + 75 ) );
				else
					controller.addFire( getPort() + (i + 10), getRange(), point3 );
				return;
			}

			room1 = controller.getChartComp().isRoom( point4 );
			if( room1 != null && !rooms.contains( room1 ) )
			{
				if( controller.getChartComp().isRoom( new Point( point4.x, point4.y - 75 ) ) == room1 )
					controller.addFire( getPort() + (i + 10), getRange(), new Point( point4.x, point4.y - 75 ) );
				else
					controller.addFire( getPort() + (i + 10), getRange(), point4 );
				return;
			}

			point1 = new Point( point1.x + 50, point1.y );
			point2 = new Point( point2.x - 50, point2.y );
			point3 = new Point( point3.x, point3.y + 50 );
			point4 = new Point( point4.x, point4.y - 50 );

        } //for

    } //method


	// Use this thread to communicate to other robots.
	class Calling extends Thread{

		public void run(){

			try{ Thread.sleep(2000); }catch(Exception e){}
			int k = 0;

			while(true){

				if( kill )
					return;

				try{ Thread.sleep(3000); }catch(Exception e){}
				k += 3;

				if( k >= 21 ){
					k = 0;
					Vector rooms = new Vector();
					Vector users = controller.getUsers();
					for( int i = 0; i < users.size(); ++i )
					{
						User user1 = (User) users.elementAt( i );

						if( user1 instanceof Fire && controller.getChartComp().isRoom( user1.getLocation() ) != null )
							rooms.add( controller.getChartComp().isRoom( user1.getLocation() ) );
					}
					k = 0;
					spread( rooms );


				}

				//if get message from firefighter
				//then we hurt him if close enough!
				Vector users = controller.getUsers();
				User u;
				for( int i=0; i< users.size(); i++){
					u = ((User)users.elementAt(i));

					//We have a person, so check range and hurt
					if(  !(u instanceof Fire) ){

						double diff = Controller.diff( getPoint(), u.getLocation() );

						if( diff < getRange() && u.getFloor() == getFloor() ){

							//Then hurt the user to some degree
							int health = u.getHealth();
							health -= Math.round( 10*(1.5-(diff/getRange())) );
							u.setHealth(health);
							controller.fireControllerChange();

							//Delete if health is zero
							if( u.getHealth() < 0 ){
								SocketSender ss = new SocketSender(new DeathMessage(u.getPort(), getPort()));
								ss.start();
								try{ Thread.sleep(500); }catch(Exception e){}
								u.setDead(true);
								controller.fireControllerChange();
							}
						}//if

					} //if

				} //for


				//actuateMessage(new FireMessage(getHealth(), getRange(), getPort(), getPoint() ));

			}

		}

	}



} //class: Rescuer